about summary refs log tree commit diff
path: root/sysdeps/s390/s390-64/bcopy.S
diff options
context:
space:
mode:
Diffstat (limited to 'sysdeps/s390/s390-64/bcopy.S')
-rw-r--r--sysdeps/s390/s390-64/bcopy.S47
1 files changed, 26 insertions, 21 deletions
diff --git a/sysdeps/s390/s390-64/bcopy.S b/sysdeps/s390/s390-64/bcopy.S
index 5cb02b3162..0d75d3ea9f 100644
--- a/sysdeps/s390/s390-64/bcopy.S
+++ b/sysdeps/s390/s390-64/bcopy.S
@@ -1,6 +1,6 @@
 /* bcopy -- copy a block from source to destination.  64 bit S/390 version.
    This file is part of the GNU C Library.
-   Copyright (C) 2001 Free Software Foundation, Inc.
+   Copyright (C) 2000, 2001 Free Software Foundation, Inc.
    Contributed by Martin Schwidefsky (schwidefsky@de.ibm.com).
 
    The GNU C Library is free software  ; you can redistribute it and/or
@@ -26,27 +26,32 @@
 #include "sysdep.h"
 #include "asm-syntax.h"
 
-        .text
+	.text
 ENTRY(__bcopy)
-        clgr    %r2,%r3             # check against destructive overlap
-        jnl     .L0
-        lgr     %r1,%r2
-        algr    %r1,%r4
-        clgr    %r1,%r3
-        jh      .L2
-.L0:
-	lgr     %r5,%r4		    # source length
-        lgr     %r4,%r2             # source address
-        sgr     %r1,%r1             # set pad byte to zero
-	lgr     %r2,%r3             # set destination
-        lgr     %r3,%r5             # destination length = source length
-.L1:    mvcle   %r2,%r4,0(%r1)      # thats it, MVCLE is your friend
-        jo      .L1
-        br      %r14
-.L2:                                # destructive overlay, can not use mvcle
-        lgr    %r1,%r2              # bcopy is called with source,dest
-        lgr    %r2,%r3              # memmove with dest,source! Oh, well...
-        lgr    %r3,%r1
+	ltgr	%r1,%r4		    # zero bcopy ?
+	jz	.L4
+	clgr	%r2,%r3		    # check against destructive overlap
+	jnl	.L0
+	algr	%r1,%r2
+	clgr	%r1,%r3
+	jh	.L5
+.L0:	aghi	%r4,-1		    # length - 1
+	srlg	%r1,%r4,8
+	ltgr	%r1,%r1
+	jz	.L2
+.L1:	mvc	0(256,%r3),0(%r2)   # move in 256 byte chunks
+	la	%r2,256(%r2)
+	la	%r3,256(%r3)
+	brctg	%r1,.L1
+.L2:	bras	%r1,.L3		    # setup base pointer for execute
+	mvc	0(1,%r3),0(%r2)	    # instruction for execute
+.L3:	ex	%r4,0(%r1)	    # execute mvc with length ((%r4)&255)+1
+.L4:	br	%r14
+
+.L5:				    # destructive overlay, can not use mvcle
+	lgr    %r1,%r2		    # bcopy is called with source,dest
+	lgr    %r2,%r3		    # memmove with dest,source! Oh, well...
+	lgr    %r3,%r1
 #ifdef PIC
 	jg     memmove@PLT
 #else